/*
 * ela_daemon.c
 *
 * (C) Copyright IBM Corp. 2004
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 *    Author: Rajarshi Das <rajarshi@in.ibm.com>
 *
 * Description: Daemon for the provider library to interface with linux 
 * event log analysis report generator
 */

# include "indprovider.h"

static int maxsd;
fd_set all_sds;
static int connfd;

struct sockaddr_un cliadd,servadd;
int clilen,listenfd,childpid,ret_bind,tempfd,status;
static struct sigaction sig_hdler,sig_hdler1;
static struct sigaction sig_term; 
sigset_t newmask,oldmask,childmask;

void closesocket(int sd)
{
	if (maxsd == sd) {
		maxsd --;
	}
	
	close(connfd);
	connfd=0;
	fprintf(stderr,"\nsocket closed . connfd : %d, mxsd = %d", 
			connfd, maxsd); // debug
}
void echobak(pid_t globpid,int sockfd,char sigtype[])
{
	int n,written;
	pid_t prevglobpid;
	char buf[200],buf1[200];
	
	buf[0] = '\0';
	buf1[0] = '\0';
	snprintf(buf,sizeof(buf),"%d%s",globpid,":"); // we want the colon to delimit...
	snprintf(buf1,sizeof(buf1),"%s%s",buf,sigtype);
	printf("\nwriting the recid of the ela report to the socket !");
	printf("\nattempting to write to descriptor:%d",sockfd);
	n = strlen(buf1);
	if ((written = write(sockfd,(char *) &n,sizeof(n))) > 0) // sending the length of data
	{
		fprintf(stderr,"\nwritten back %d byte containing the length of following data.",written);
		written = 0;
	}
	else 
	{
		perror("\nwrite");
		fprintf(stderr,"\nwritten :%d",written);
		closesocket(sockfd);
		return;
	}

	if ((written = write(sockfd,buf1,n)) > 0) // sending the data itself
	{
		fprintf(stderr,"\nbytes of data written : %d",written);
	}
	else 
	{
		perror("\nwrite");
		fprintf(stderr,"\nwritten :%d",written);
		closesocket(sockfd);
	}
}


int d_init()
{
	pid_t pid;

	if ((pid = fork()) < 0) 
	{
		printf("error !");
		return -1;
	}
        else if (pid >0)
	                exit(0);
	     else 
	     {
	               printf("\nstarted a daemon !");
	               setsid(); // become a session leader.
	               umask(0); // clear file creation mask.
	               return 0;
	     }
}


static void
sig_handler(int signo, siginfo_t *info, void *context)
{
	char sig_type[10];
        int recid = info->si_errno;
        int sub_id = info->_sifields._rt.si_sigval.sival_int;
        fprintf(stdout, "\nrecid = %d, sub id = %d.\n", recid, sub_id);
	if (signo == SIGRTMIN+15)
		snprintf(sig_type,sizeof(sig_type),"%s","ioela");
	
	else if (signo == SIGRTMIN+14)
		snprintf(sig_type,sizeof(sig_type),"%s","pfela");

	if (connfd > 0)	{
		echobak(recid,connfd,sig_type);
	}
	else {
		fprintf(stderr,"\nfd not available:cannot write data!");
		closesocket(connfd);
	}
		
}

static void sig_pipe(int signo)
{
	int filed;

	fprintf(stderr,"probably the other end closed!");
//	close(connfd);
	fprintf(stderr,"closed connfd: %d",connfd);
//	daemon_accept();
}

static void sig_handler1(int signo)
{
	fprintf(stderr,"within handler for sigrtmax-15");
	
//	close(connfd);
	fprintf(stderr,"\n closed connfd: %d",connfd);	
//	daemon_accept();
}


int main(int argc, char *argv[])
{
	fd_set read_sds;
	d_init(); // initialising the daemon .
	
	(void) memset(&sig_hdler, 0, sizeof(sig_hdler));
	sig_hdler.sa_handler = (void *) sig_handler;
	sig_hdler.sa_flags = SA_SIGINFO;                        

	if (sigaction(SIGRTMIN+15, &sig_hdler, NULL) < 0)
	{
	      (void)fprintf(stderr, "sig_rev: sigaction failed for SIGRTMIN+15.\n");
              perror("sigaction");
	}

	if (sigaction(SIGRTMIN+14, &sig_hdler, NULL) < 0)
	{
	      (void)fprintf(stderr, "sig_rev: sigaction failed for SIGRTMIN+14.\n");
              perror("sigaction");
	}

	(void) memset(&sig_hdler1, 0, sizeof(sig_hdler1));
	sig_hdler1.sa_handler = (void *) sig_handler1;
	if (sigaction(SIGRTMAX-15,&sig_hdler1,NULL) < 0)
	{
	      (void)fprintf(stderr, "sig_rev: sigaction failed for new SIGRTMIN.\n");
              perror("sigaction");
	}
	
	if ((listenfd = socket(AF_UNIX,SOCK_STREAM,0)) < 0)
		perror("\nsocket");

	signal(SIGPIPE,sig_pipe);
	unlink("/var/run/elasocket");
	bzero(&servadd,sizeof(servadd));
	servadd.sun_family = AF_UNIX;
	snprintf(servadd.sun_path,sizeof(servadd.sun_path),"/var/run/elasocket");
	printf("set the socket path to : %s",servadd.sun_path);

	if ((ret_bind = bind(listenfd, (struct sockaddr*) &servadd, sizeof(servadd))) == 0)
		printf("\nbound successfully to socket");
	else perror("\nbind"),printf("\nerror in bind : %d",errno);

	
	listen(listenfd,1024);
	FD_ZERO(&all_sds);
	FD_SET(listenfd, &all_sds);
	maxsd = listenfd;
	fprintf(stderr,"\nconnfd : %d",connfd); // debug
	for(;;) {
		int newsd;
		int nsel;
		
		bcopy((char *)&all_sds, (char *)&read_sds, sizeof(all_sds));
		if ((nsel = select(maxsd + 1, &read_sds, 0, 0, 0)) < 0) {
			if (errno == EINTR) {
				continue;
			} else if (errno != EBADF) {
				exit(1);
			}
		}
		if (nsel == 0) {
			/* timeout */
			continue;
		}
		if (FD_ISSET(listenfd, &read_sds)) {
			/* provider tries to connect */
			if ((newsd = accept(listenfd, NULL, NULL)) < 0) {
				perror("accept");
				exit(1);
			}
			FD_SET(newsd, &all_sds);
			if (maxsd < newsd) {
				maxsd = newsd;
			}
			
			connfd = newsd;
			fprintf(stderr,"\nconnfd : %d",connfd); // debug
		}
		if (FD_ISSET(connfd, &read_sds)) {
			/* connection goes away */
			closesocket(connfd);
		}  
		
	} /* end for */
	
	return 0;
}
