/*
 * elaindicationprovider.c
 *
 * (C) Copyright IBM Corp 2004
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *            
 *    Author: Rajarshi Das <rajarshi@in.ibm.com>
 *              
 * Interface Type: Common Manageability Programming Interface ( CMPI )
 *
 * Description: CIM Provider for Event Log Analysis
 *
 */
/*Note:
 * the cimom server needs to be restarted only after deleting the subscriptions
 * explicitly if it needs to be restarted without starting the daemon first. Set 
 *  at the perror code
 *  the srn error text is supposed to be in a single line. If it exceeds more than
 * a line, it is not read in. 
 */

#include "indprovider.h"

#define CMPI_VER_86 1

#include "cmpidt.h"
#include "cmpift.h"

#include "cmpimacs.h"

unsigned char CMPI_true = 1;
unsigned char CMPI_false = 0;

int procSubscr = 0;
int creaSubscr = 0;
int indReady = 1;


CMPIInstance *instModification = NULL, *instDeletion = NULL, *instCreation =
  NULL, *instProcess = NULL;

int dataNext = 0;

CMPIBroker *broker;

static void
sig_term (int signo)
{
  system ("/usr/bin/sig_ela_daemon.sh");
  signal (SIGTERM, SIG_DFL);
  raise (SIGTERM);
}

static void
sig_int (int signo)
{
  system ("/usr/bin/sig_ela_daemon.sh");
  signal (SIGINT, SIG_DFL);
  raise (SIGINT);
}

void *
timerThread (void *arg)
{
  struct sockaddr_un servadd;
  struct dirent **nl;
  int sockfd=0, n, r, ret;
  char recid[200], buf[200], buf1[200], buf2[200], buf3[100],
    elarecpidbuf[200], hname[50];
  elareport *ela_r;
  char signaltype[10];
  char *tokbuf[100], *temp;
  void **args = (void **) arg;
  CMPIBroker *broker = (CMPIBroker *) args[0];
  CMPIContext *ctx = (CMPIContext *) args[1];
  long interval = (long) args[2];
  CMPIStatus stat;

  fd_set all_sds, read_sds;
  int maxsd;

  fprintf (stderr,
	   "--- elaindicationprovider::TimerThread() timer thread interval=%lds\n",
	   interval);

  CBAttachThread (broker, ctx);
  fprintf (stderr, "--- elaindicationprovider::TimerThread(1)\n");
  fprintf (stderr, "--- elaindicationprovider::TimerThread(2)\n");

  fprintf (stderr,
	   "--- elaindicationprovider::timerThread() thread attached\n");

  sockfd = socket (AF_UNIX, SOCK_STREAM, 0);
  bzero (&servadd, sizeof (servadd));
  servadd.sun_family = AF_UNIX;
  snprintf (servadd.sun_path, sizeof (servadd.sun_path),
	    "/var/run/elasocket");
  fprintf (stderr, "\nsockfd : %d", sockfd);
  signal (SIGTERM, sig_term);
  signal (SIGINT, sig_int);

  FD_ZERO (&all_sds);
  FD_ZERO (&read_sds);
  FD_SET(sockfd, &all_sds);
  maxsd = sockfd;

  if ((ret =
       connect (sockfd, (struct sockaddr *) &servadd, sizeof (servadd))) == 0)
    {
      fprintf (stderr, "\n connect succeeded !");
      for (;;)
	{
	  int nsel;
	  int len;
	  struct timeval tv, *ptv;

	  if (sockfd <= 0) {
		fprintf(stderr, "Trying to connect to ela_daemon\n");
 		sockfd = socket (AF_UNIX, SOCK_STREAM, 0);
  		bzero (&servadd, sizeof (servadd));
  		servadd.sun_family = AF_UNIX;
  		snprintf (servadd.sun_path, sizeof (servadd.sun_path),
            		"/var/run/elasocket");

		if (connect (sockfd, (struct sockaddr *) &servadd, 
			sizeof (servadd)) < 0) {
			close(sockfd);
			sockfd = 0;
			tv.tv_sec = 3;
			tv.tv_usec = 0;
			ptv = &tv;
		} 
		else {
			FD_SET(sockfd, &all_sds);
			maxsd = sockfd;
			ptv = NULL;			  
                                                                              
		}
	  }	

	  bcopy ((char *) &all_sds, (char *) &read_sds, sizeof (all_sds));
	  if ((nsel = select (maxsd + 1, &read_sds, 0, 0, ptv)) < 0)
	    {
	      if (errno == EINTR)
		{
		  continue;
		}
	      else if (errno != EBADF)
		{
		  exit (1);
		}
	    }
	  if (nsel == 0)
	    {
	      /* timeout */
	      continue;
	    }
	  fprintf (stderr, "\nnsel is : %d", nsel);	// debug
	  if (FD_ISSET (sockfd, &read_sds))
	    {
	      /* we have data - first read the size */
	      if ((r =
		   read (sockfd, (char *) &len, sizeof (len))) != sizeof(len))
		{
		  close (sockfd);
		  if (maxsd == sockfd) maxsd--;
		  sockfd = 0;
		  continue;
		}
	      fprintf (stderr, "\nread len= %d :", len);
	      /* then read the data */
	      if ((r = read (sockfd, buf1, len)) != len)
		{
		  close (sockfd);
		  if (maxsd == sockfd) maxsd--;
		  sockfd = 0;
		  continue;
		}
	      fprintf (stderr, "\nread %d bytes :", r);
	      if (r > 0)
		{
		  if ((strstr (buf1, "ioela") != NULL)
		      || (strstr (buf1, "pfela") != NULL))
		    {
		      temp = strtok_r (buf1, ":", tokbuf);
		      snprintf (recid, sizeof(recid), "%s", temp);
		      temp[0] = '\0';
		      temp = strtok_r (NULL, ":", tokbuf);
		      snprintf (signaltype, sizeof (signaltype), "%s", temp);
		      tokbuf[0][0] = '\0';
		      fprintf (stderr, "\nrecid : %s", recid);
		      fprintf (stderr, "\nsignaltype : %s", signaltype);

		      if (strncmp (signaltype, "ioela", 5) == 0)	// io ela
			n = scandir (ELA_REPORT_DIR, &nl, 0, alphasort);
		      else if (strncmp (signaltype, "pfela", 5) == 0)	// platform ela
			n =
			  scandir (PLATFORM_ELA_DATA_DIR, &nl, 0, alphasort);
		      if (n < 0)
			perror ("\nscandir");
		      else
			{
			  while (n--)
			    {
			      if ((strstr (nl[n]->d_name, recid)) != NULL)
				{
				  snprintf (buf3, sizeof (buf3),
					    "ela_report%s.", recid);
				  if ((strstr (nl[n]->d_name, buf3)) != NULL)
				    {
				      snprintf (buf2, sizeof (buf2), "%s",
						nl[n]->d_name);
				      buf3[0] = '\0';
				    }
				  else
				    {	// the recid passed is a subset of one of the report recids 
				      buf[3] = '\0';
				      continue;
				    }
				}
			      else
				{
				}
			      // do nothing ..
			      free (nl[n]);
			    }
			  if (strcmp (buf2, "") == 0)
			    fprintf (stderr,
				     "\nno ela report with recid %s was created !",
				     recid);
			}
		      free (nl);

		      if (strcmp (buf2, "") != 0)
			{
			  if (strncmp (signaltype, "ioela", 5) == 0)	// io ela
			    snprintf (elarecpidbuf, sizeof (elarecpidbuf),
				      ELA_REPORT_DIR "/%s", buf2);
			  else if (strncmp (signaltype, "pfela", 5) == 0)	// platform ela
			    snprintf (elarecpidbuf, sizeof (elarecpidbuf),
				      PLATFORM_ELA_DATA_DIR "/%s", buf2);
			  ela_r = malloc (sizeof (elareport));
			  fill_ela_report (elarecpidbuf, ela_r);	// read file and fill structure
			  fprintf (stderr, "ela report filled!");	// debug 
			  if (gethostname (hname, sizeof (hname)) == 0)
			    {
			      fprintf (stdout, "---hostname=%s\n", hname);
			    }
			  else
			    {
			      fprintf (stdout, "gethostname failed \n");
			      strcpy (hname, "unknown");
			    }
			  CMSetProperty (instProcess, "hostname",
					 (CMPIValue *) & hname, CMPI_chars);
			  hname[0] = '\0';
			  CMSetProperty (instProcess, "message",
					 (CMPIValue *) "process indication",
					 CMPI_chars);
			  fprintf (stderr, "ela_r->datetime : %s",
				   ela_r->datetime);
			  CMSetProperty (instProcess, "timestamp",	//replaced 'datetime'
					 (CMPIValue *) ela_r->datetime,
					 CMPI_chars);
			  CMSetProperty (instProcess, "srn",
					 (CMPIValue *) ela_r->srn,
					 CMPI_chars);
			  CMSetProperty (instProcess, "srnerrortext",
					 (CMPIValue *) ela_r->srnerrortext,
					 CMPI_chars);
			  CMSetProperty (instProcess, "menunumber",
					 (CMPIValue *) ela_r->menunumber,
					 CMPI_chars);
			  CMSetProperty (instProcess, "menugoalerrortext",
					 (CMPIValue *) ela_r->
					 menugoalerrortext, CMPI_chars);
			  CMSetProperty (instProcess, "location",
					 (CMPIValue *) ela_r->location,
					 CMPI_chars);
			  CMSetProperty (instProcess, "partnumber",
					 (CMPIValue *) ela_r->partnumber,
					 CMPI_chars);
			  CMSetProperty (instProcess, "errlogseqno",
					 (CMPIValue *) ela_r->errorlogseqno,
					 CMPI_chars);
			  stat =
			    CBDeliverIndication (broker, ctx, "root/cimv2",
						 instProcess);
			  if (stat.rc == CMPI_RC_OK)
			    {
			      fprintf (stderr,
				       "---Delivered indication\n");
			    }
			  else
			    {
			      fprintf (stderr,
				       "--- Failed to deliver indication.\n");
			    }
			  free (ela_r);
			}
		      elarecpidbuf[0] = '\0';
		      buf2[0] = '\0';
		      recid[0] = '\0';
		      signaltype[0] = '\0';
		    }		// end of if "ioela" or "pfela" is found...
		  else
		    {
		      fprintf (stderr,
			       "cannot service data: either the record id or"
			       "the signal type is (or both are) incorrect!");
		      buf1[0] = '\0';
		    }
		}
	      else
		{
		  fprintf (stderr,
			   "1) if the server alone was explicitly terminated, ");
		  fprintf (stderr,
			   "then the daemon is still alive and the server just");
		  fprintf (stderr, " needs to be started again,else , ");
		  fprintf (stderr,
			   "2) the daemon has gone down because it was ");
		  fprintf (stderr, "explicitly terminated thus rendering ");
		  fprintf (stderr, ",existing subscriptions ineffective,");
		  fprintf (stderr,
			   " hence, delete all existing subscriptions");
		  fprintf (stderr, " ,restart the daemon, and then create ");
		  fprintf (stderr, " a new subscription!");
		  ret = close (sockfd);
		  CBDetachThread (broker, ctx);
		  pthread_exit ((void *) 1);
		}

	    }			/* end of if FD_ISSET */
	  else
	    {
	      ret = close (sockfd);
	      if (maxsd == sockfd) maxsd--;
	      sockfd = 0;
	      fprintf (stderr, "FD_ISSET for sockfd failed !");
	    }
	}			/* end for */
    }
  else
    {
      ret = close (sockfd);
      fprintf (stderr, "\nreturn value from close : %d", ret);
      fprintf (stderr, "the connect has failed since ");
      fprintf (stderr, "the daemon is not running!");
      fprintf (stderr, " please delete existing subscriptions if any ");
      fprintf (stderr, ",and then start the daemon and ");
      fprintf (stderr, "create subscriptions!");
      CBDetachThread (broker, ctx);
      pthread_exit ((void *) 1);
      return 0;
    }
}

int
startTimerThread (CMPIBroker * broker, CMPIContext * ctx, int pulse)
{
  pthread_t thread;
  void **args = (void **) malloc (3 * sizeof (char *));
  fprintf (stderr, "---TimerThread() starting (%p)...\n", broker);
  args[0] = broker;
  args[1] = CBPrepareAttachThread (broker, ctx);
  args[2] = (void *) pulse;
  pthread_create (&thread, NULL, timerThread, args);
  return 0;
}

int
elaindicationproviderClassesLoaded (CMPIBroker * broker)
{
  CMPIObjectPath *op;
  CMPIStatus rc;

  fprintf (stderr, "+++ within ProviderClassesLoaded !");

  if (instProcess != NULL)
    return 1;
  if (indReady == 0)
    return 0;

  for (;;)
    {
      op = CMNewObjectPath (broker, "root/cimv2", "CIM_InstCreation", &rc);
      instCreation = CMNewInstance (broker, op, &rc);
      if (rc.rc)
	{
	  fprintf (stderr, "problem loading cim_instcreation !");
	  break;
	}
      instCreation = CMClone (instCreation, &rc);

      CMSetClassName (op, "Ela_ReportIndication");
      instProcess = CMNewInstance (broker, op, &rc);
      if (rc.rc)
	{
	  instProcess = NULL;
	  fprintf (stderr, "problem loading ela_reportindication !");
	  break;
	}
      instProcess = CMClone (instProcess, &rc);
      fprintf (stderr,
	       "--- elaindicationprovider::initialize(): life-cyle indication classes loaded\n");
      return 1;
    }

  fprintf (stderr,
	   "--- elaindicationprovider::initialize(): Could not get life-cyle indication classes\n");
  indReady = 0;
  return 0;
}

CMPIStatus
elaindicationproviderCleanup (CMPIInstanceMI * cThis, CMPIContext * ctx)
{
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderEnumInstanceNames
  (CMPIInstanceMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPIObjectPath * ref)
{
  CMReturn (CMPI_RC_OK);
}


CMPIStatus elaindicationproviderEnumInstances
  (CMPIInstanceMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPIObjectPath * ref, char **properties)
{
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderGetInstance
  (CMPIInstanceMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPIObjectPath * cop, char **properties)
{
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderCreateInstance
  (CMPIInstanceMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPIObjectPath * cop, CMPIInstance * inst)
{
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderSetInstance
  (CMPIInstanceMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPIObjectPath * cop, CMPIInstance * inst, char **properties)
{
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderDeleteInstance
  (CMPIInstanceMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPIObjectPath * cop)
{
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderExecQuery
  (CMPIInstanceMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPIObjectPath * cop, char *lang, char *query)
{
  CMReturn (CMPI_RC_ERR_NOT_SUPPORTED);
}




//----------------------------------------------------------
//---
//      Indication Provider
//---
//----------------------------------------------------------


CMPIStatus
elaindicationproviderIndicationCleanup (CMPIIndicationMI * cThis,
					CMPIContext * ctx)
{
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderAuthorizeFilter
  (CMPIIndicationMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPISelectExp * filter, char *indType, CMPIObjectPath * classPath,
   char *owner)
{
  fprintf (stderr, "--- indAuthorizeFilter(): %s\n", indType);
  if (strcasecmp (indType, "Ela_ReportIndication") == 0)
    {
      CMReturnData (rslt, (CMPIValue *) & CMPI_true, CMPI_boolean);
      CMReturnDone (rslt);
    }
  else
    CMReturnData (rslt, (CMPIValue *) & CMPI_false, CMPI_boolean);
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderMustPoll
  (CMPIIndicationMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPISelectExp * filter, char *indType, CMPIObjectPath * classPath)
{
  fprintf (stderr, "--- indMustPoll(): \n");
  CMReturnData (rslt, (CMPIValue *) & CMPI_false, CMPI_boolean);
  CMReturnDone (rslt);
  CMReturn (CMPI_RC_OK);
}

CMPIStatus elaindicationproviderActivateFilter
  (CMPIIndicationMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPISelectExp * filter, char *indType, CMPIObjectPath * classPath,
   CMPIBoolean firstActivation)
{
  CMPIStatus rc;
  CMPIString *fltr;

  fprintf (stderr, "--- activateFilter() indication type: %s\n", indType);
  fltr = CMGetSelExpString (filter, &rc);
  fprintf (stderr, "--- activateFilter() filter: %s\n", (char *) fltr->hdl);

  if (strcasecmp (indType, "Ela_ReportIndication") == 0)
    {
      if (elaindicationproviderClassesLoaded (broker))
	procSubscr++;

      if (procSubscr == 1)
	startTimerThread (broker, ctx, 5);
    }
  else
    fprintf (stderr, "\nindication type doesnot match Ela_ReportIndication!");
  CMReturn (CMPI_RC_OK);
}


CMPIStatus elaindicationproviderDeActivateFilter
  (CMPIIndicationMI * cThis, CMPIContext * ctx, CMPIResult * rslt,
   CMPISelectExp * filter, char *indType, CMPIObjectPath * classPath,
   CMPIBoolean lastActivation)
{
  fprintf (stderr, "---deActivateFilter()\n");

  if (strcasecmp (indType, "Ela_ReportIndication") == 0)
    {
      if (procSubscr > 0)
	procSubscr--;
    }
  CMReturn (CMPI_RC_OK);
}

void
elaindicationproviderEnableIndications (CMPIIndicationMI * cThis)
{
  fprintf (stderr, "+++ elaindicationproviderEnableIndications\n");
}

void
elaindicationproviderDisableIndications (CMPIIndicationMI * cThis)
{
  fprintf (stderr, "+++ elaindicationproviderDisableIndications\n");
}


//----------------------------------------------------------
//---
//      Provider Factory Stubs
//---
//----------------------------------------------------------

CMInstanceMIStub (elaindicationprovider, elaindicationprovider, broker,
		  CMNoHook);

//----------------------------------------------------------

CMIndicationMIStub (elaindicationprovider, elaindicationprovider, broker,
		    CMNoHook);
