/*
 * IndPropertyResolve.java
 *
 * (C) Copyright IBM Corp. 2002 2004
 * 
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author: Viktor Mihajlovski <MIHAJLOV@de.ibm.com>
 * Contributors:
		Rajarshi Das <rajarshi@in.ibm.com>
 *
 * Description:  Auxiliary class that can be used for parameterized 
 *               Indication Actions. This file has been derived from the original
		 file IndPropertyResolver.java. The original file has been modified 
		to represent event log analysis report. It substitutes the tokens 
		%h %t, %s, %e,%m %x %l %p %g
 */

package com.ibm.wbem.evsub;

import org.snia.wbem.cim.*;
import org.snia.wbem.client.*;

public class IndPropertyResolve {
    static class Bounds {
	Bounds() {from=0; to=0; last=0;}
	int from;
	int to;
	int last;
    }
    private static void locateBounds(String format, Bounds bounds) {
	bounds.last=bounds.to;
	bounds.from=format.indexOf("%",bounds.to);
	if (bounds.from>-1) {
	    bounds.to=format.indexOf(" ",bounds.from);
	    if (bounds.to<0) bounds.to=format.length();
	}
    }
    private static String replace(String token, CIMEvent event) {
	boolean srn=false;
	boolean datetime=false;
	boolean hostname=false;
	boolean srnerrortext=false;
	boolean menunumber=false;
	boolean menugoalerrortext=false;
	boolean partnumber=false;
	boolean location=false;
	boolean errlogseqno=false;
	switch (token.charAt(1)) {
	case 'h':
	    hostname=true;
	    break;
	case 's':
	    srn=true;
	    break;
	case 'e':
	    srnerrortext=true;
	    break;
	case 'm':
	    menunumber=true;
	    break;
	case 'x':
	    menugoalerrortext=true;
	    break;
	case 'p':
	    partnumber=true;
	    break;
	case 't':
	    datetime=true;
	    break;
	case 'l':
	    location=true;
	    break;
	case 'g':
	    errlogseqno=true;
	    break;
	default:
	    return "!!! Invalid format string";
	}
	CIMInstance evinst=event.getIndication();
	CIMInstance inst=null;
	int propindex;
	String propertyname=null;
	if (hostname) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst = evinst;
	    propertyname="hostname";
	};
	if (srn) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="srn";
	};
	if (srnerrortext) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="srnerrortext";
	};
	if (menunumber) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="menunumber";
	};
	if (menugoalerrortext) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="menugoalerrortext";
	};
	if (partnumber) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="partnumber";
	};
	if (datetime) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="timestamp";
	};
	if (location) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="location";
	};
	if (errlogseqno) {
	    propindex=token.indexOf(".");
	    if (propindex<0)
		return "!!! Invalid format string";
	    inst=evinst;
	    propertyname="errlogseqno";
	};
	CIMValue resolved=inst.getProperty(propertyname).getValue();
	String stringresolved="";
	if (resolved!=null)
	    stringresolved=resolved.getValue().toString();
	else stringresolved="<n/a>";
	return stringresolved;
    }
    public static String resolve(String format, CIMEvent event) {
	String result = "";
	Bounds resolverBounds = new Bounds();
	locateBounds(format,resolverBounds);
	result = System.getProperty("line.separator");
	while (resolverBounds.from > -1) {
	    result = result +
		format.substring(resolverBounds.last,resolverBounds.from) +
		replace(format.substring(resolverBounds.from,
					 resolverBounds.to),
			event) + System.getProperty("line.separator");
	    locateBounds(format,resolverBounds);
	}
	if (resolverBounds.last != format.length())
	    result = result + format.substring(resolverBounds.last);
	return result;
    }
}
