/*
 * IndicationArbiter.java
 *
 * (C) Copyright IBM Corp. 2002 2004
 * 
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author: Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors: Rajarshi Das <rajarshi@in.ibm.com>
 *
 * Description: The handler engine, it connects to a client and offers a
 *              subscription interface.
 */

package com.ibm.wbem.evsub;

import org.snia.wbem.cim.*;
import org.snia.wbem.client.*;
import java.util.*;
import java.io.*; 
public class IndicationArbiter {
    /*
     * Handler class
     */
    class ArbiterHandler extends CIMHTTPListener {
	IndConsumer handlerConsumer;
	ArbiterHandler(String index, IndConsumer consumer) {
	    super("IndicationArbiter " + index, 
		  "CIM_IndicationHandlerCIMXML");
	    handlerConsumer=consumer; 
	System.out.println("\nnew indication arbiter called !");
	}
	public void indicationOccured(CIMEvent ce) {
	    handlerConsumer.processIndication(ce);
	}
    }
    /*
     * fields
     */
    CIMClient       client;
    int             filterindex=0;
    /*
     * constructors
     */
    public IndicationArbiter(CIMClient cc) throws CIMException {
	client=cc;
    }

    public IndicationArbiter(CIMClient cc,String flag) throws CIMException {
	client=cc;
	cancel(flag);
    }

    public void subscribe(IndSubscription subscription)
	throws CIMException {
	String procid = null;
	/* setting the pid to the index */
	procid = getpid_current();
	CIMHTTPListener listener = 
	    new ArbiterHandler(procid,subscription.consumer);
	client.addCIMListener(listener);
	System.out.println("\nlistener started !");
	
	CIMObjectPath filterPath = addFilter(subscription);
	System.out.println("\nThe filter expression path is :" + filterPath);
	CIMClass cls = 
	    client.getClass(new CIMObjectPath("CIM_IndicationSubscription"),
			    true);
	CIMInstance cis = cls.newInstance();
	cis.setProperty("Filter", new CIMValue(filterPath));
	cis.setProperty("Handler", new CIMValue(listener.getHandlerPath()));
	client.createInstance(new CIMObjectPath(cis.getClassName(),
						cis.getKeyValuePairs()),
			      cis);
    }
    public void cancel(String flag) throws CIMException {
	Process proc1 = null, proc2 = null, proc3 = null;
	Enumeration handlers = null;
	FileInputStream fis = null;	
	File fil = null,fil1 = null;
	int noofsubs = 0,clientcount = 0, childcount = 0, relpos = 0;	
	String pid = null, pid1 = null, pid2 = null;
	long len;
	int temp,found;
	Integer a = null;
	
	String check = new String();
	check = "cleanrepos";
	if (flag.equals(check)) // this only cleans all existing subs. doesnt terminate
				// createsub.sh processes and their children
	{
	handlers = 
	    client.execQuery(new CIMObjectPath("CIM_IndicationHandlerCIMXML"),
			     "select * from CIM_IndicationHandlerCIMXML " +
			     " where name like \"IndicationArbiter%\"",2);
	
	while(handlers.hasMoreElements()) {
	    noofsubs++;
	    System.out.println("subscription count" + noofsubs);
	    CIMInstance ci = (CIMInstance)handlers.nextElement();
	    System.out.println("Instance : " + ci.toString());
	    CIMObjectPath coph = new CIMObjectPath(ci.getClassName(), 
						   ci.getKeyValuePairs());
	    System.out.println("Found active handler - removing it");
	    Enumeration subscriptions = 
		client.referenceNames(coph,"CIM_IndicationSubscription",
				      null);
	    Enumeration filters = 
		client.associatorNames(coph,"CIM_IndicationSubscription",
				       null,null,null);
	    while (subscriptions.hasMoreElements()) {
		CIMObjectPath cops = 
		    (CIMObjectPath)subscriptions.nextElement();
		System.out.println("cancel subscription " + cops);
		client.deleteInstance(cops);
	    }
	    while (filters.hasMoreElements()) {
		CIMObjectPath copf = 
		    (CIMObjectPath)filters.nextElement();
		System.out.println("cancel filter " + copf);
		client.deleteInstance(copf);
	    }
	    client.deleteInstance(coph);
	} // end of while
	} // end of flag = "cleanrepos"
	else
	{ 
	// need to parse the pid here. 
		pid = new String(flag);
		pid1 = new String();
		pid2 = new String();
		System.out.println("pid set !");
		System.out.println("pid:" + pid);
		try 
		{
		 pid1 = "kill -s SIGTERM " + pid;
		 proc1 = (Runtime.getRuntime()).exec(pid1);
		 temp = a.parseInt(pid);
		 System.out.println("temp :" + temp);
		 temp = temp + 1;
		 pid1 = pid.valueOf(temp);
		 pid2 = null;
		 pid2 = "kill -s SIGTERM " + pid1;
		 proc2 = (Runtime.getRuntime()).exec(pid2);
		 proc1 = null;
		 proc2 = null;
		 temp = 0;
		 pid1 = null;
		 }
		catch (NumberFormatException e)
		{
			System.out.println("An incorrect parameter was passed to cancelsub.sh");
			System.out.println("please specify either <cleanrepos>/<pid of ");
			System.out.println("createsub.sh process to be cancelled>");
		  e.printStackTrace();
	 	 (Runtime.getRuntime()).exit(1);
		}
		catch (Exception e)
		{
		  e.printStackTrace();
		}
	temp = 0;
	String deleteid = new String();
	deleteid = "IndicationArbiter " + pid;
	System.out.println("\ndeleteid : " + deleteid);
	System.out.println("length of deleteid : "+ deleteid.length());
	handlers = 
	    client.execQuery(new CIMObjectPath("CIM_IndicationHandlerCIMXML"),
			     "select * from CIM_IndicationHandlerCIMXML " +
			     " where name like \"deleteid\"",2);
	found = 0;
	while((handlers.hasMoreElements()) && (found == 0)) {
	temp = 0;

	    noofsubs++;
	    System.out.println("subscription count" + noofsubs);
	    CIMInstance ci = (CIMInstance)handlers.nextElement();
	    System.out.println("instance: " + ci.toString());
	    pid1 = ci.toString();
	try
	{
	    temp = pid1.indexOf(pid); 
	}
	catch (Exception e)
	{
	  e.printStackTrace();
	}
	    System.out.println("match : " + temp);
	    pid1 = null;
	    if (temp > 0)
	    {
	    found = 1;
	    CIMObjectPath coph = new CIMObjectPath(ci.getClassName(), 
						   ci.getKeyValuePairs());
	    System.out.println("cimobjectpath:" + coph);
	    System.out.println("Found active handler - removing it");
	    Enumeration subscriptions = 
		client.referenceNames(coph,"CIM_IndicationSubscription",
				      null);
	    Enumeration filters = 
		client.associatorNames(coph,"CIM_IndicationSubscription",
				       null,null,null);
	    while (subscriptions.hasMoreElements()) {
		CIMObjectPath cops = 
		    (CIMObjectPath)subscriptions.nextElement();
		System.out.println("cancel subscription " + cops);
		client.deleteInstance(cops);
	    }
	    while (filters.hasMoreElements()) {
		CIMObjectPath copf = 
		    (CIMObjectPath)filters.nextElement();
		System.out.println("cancel filter " + copf);
		client.deleteInstance(copf);
	    } 
	    client.deleteInstance(coph);
	    } // end of if .. temp .. 
	} // end of while .. 
	if ((found == 0) && (temp == -1)) // the pid was not found
	{
		System.out.println("\nthe pid mentioned was incorrect");
		System.out.println("\nplease specify the correct process");
		System.out.println(" id of the createsub.sh process to be cancelled");
	}
	 pid = null;
	} // end of "else" clause.
    }

    public String getpid_current() 
	throws CIMException {
	Process proc = null;
	try {
	proc = (Runtime.getRuntime()).exec("/usr/bin/curpid.sh");
	}
	catch ( Exception e )
	{
		System.out.println("Following exception thrown:" + e);
	}
	BufferedReader br = new BufferedReader(new InputStreamReader(proc.getInputStream()));
	String pid = null;
	try 
	{
		pid = br.readLine();
	}
	catch ( Exception e )
	{
		System.out.println("Following exception thrown:" + e);
	}
	return pid;	
	
    }
	
    public CIMObjectPath addFilter(IndSubscription subscription) 
	throws CIMException {
	String procid = null;
    	CIMClass cls = 
	    client.getClass(new CIMObjectPath("CIM_IndicationFilter"),true);
	System.out.println("got class");
	CIMInstance ci = cls.newInstance();
	System.out.println("got new instance");
	procid = getpid_current();
	ci.setProperty("Name", new CIMValue("ArbFilter " + procid));
	System.out.println("\nName set to ArbFilter "+ procid);
	ci.setProperty("CreationClassName", 
		       new CIMValue("CIM_IndicationFilter"));
	ci.setProperty("SystemName",new CIMValue("localhost"));
	ci.setProperty("SystemCreationClassName",
		       new CIMValue("CIM_UnitaryComputerSystem"));
	ci.setProperty("SourceNameSpace",
		       new CIMValue(subscription.namespace.getNameSpace()));
	ci.setProperty("Query", new CIMValue(subscription.filter));
	CIMObjectPath fop = 
	    client.createInstance(new CIMObjectPath(ci.getClassName(),
						ci.getKeyValuePairs()),
			      ci);
	System.out.println("created a filter instance");
	return fop;
    }
    /*
     * test program
     */
    static public void main(String args[]) {
	try {
	    CIMNameSpace cns=new CIMNameSpace(args[0],"root/cimv2");
	    CIMClient cc = new CIMClient(cns, args[1],args[2]);
	    IndicationArbiter ia = new IndicationArbiter(cc);
	    System.out.println("Starting the indication arbiter succeeded");
	    String parmsa[] = {"Handler A ","%s.description ",
			       "%p.description ", "Time: %t"};
	    ia.subscribe(new 
		IndSubscription(cns,
				"select * from cim_instmodification " +
				"where sourceInstance " +
				"isa apache_httpdirectorysetting",
				new IndActionConsumer(new PopupAction(parmsa))));
	} catch (Exception exc) {
	    System.out.println("Starting the indication arbiter failed");
	    exc.printStackTrace();
	} 
    }
}
