/*
 * ProcessSubscriptions.java
 *
 * (C) Copyright IBM Corp. 2003 2004
 * 
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author: Viktor Mihajlovski <MIHAJLOV@de.ibm.com>
 * Contributors:
 *		Rajarshi Das <rajarshi@in.ibm.com>
 * Description: This class is used to convert the subscription definitions
 *              in subscriptions.xml into IndSubscriptions. If run as
 *              standalone program it will start the event handling. 
 */

package com.ibm.wbem.evsub;

import java.io.*;
import java.util.*;

import org.snia.wbem.client.*;
import org.snia.wbem.cim.*;
import org.snia.wbem.*;
import org.apache.xerces.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

public class ProcessSubscriptions {
    private String namespace;
    private class Subscription {
	String filter;
	String handler;
	Vector args;
    }
    private Vector subscriptions;
    public void parseSubscriptions() throws Exception {
	DOMParser xmlparser = new DOMParser();
	xmlparser.parse(new InputSource(new FileInputStream("subscriptions.xml")));
	Document root=xmlparser.getDocument();
	NodeList list=root.getElementsByTagName("namespace");
	if (list.getLength() != 1)
	    throw new Exception("Need exactly one <namespace> tag ");
	NodeList textlist = list.item(0).getChildNodes();
	namespace = list.item(0).getFirstChild().getNodeValue().trim();
	list = root.getElementsByTagName("subscription");
	subscriptions = new Vector();
	for (int i=0; i<list.getLength();i++) {
	    String filter = new String();
	    String handler = new String();
	    Vector args = new Vector();
	    NodeList childnodes = list.item(i).getChildNodes();
	    for (int j=0;j<childnodes.getLength();j++) {
		if (childnodes.item(j).getNodeName().equals("filter")) {
		    filter=childnodes.item(j).getFirstChild().getNodeValue().trim();
		} else if (childnodes.item(j).getNodeName().equals("handler")) {
		    
		    NodeList handlernodes=childnodes.item(j).getChildNodes();
		    for (int k=0; k < handlernodes.getLength(); k++) {
			if (handlernodes.item(k).getNodeName().equals("type"))
			    handler=handlernodes.item(k).getFirstChild().getNodeValue().trim();
			else if (handlernodes.item(k).getNodeName().equals("arg")) 
			    args.add(handlernodes.item(k).getFirstChild().getNodeValue().trim());
		    }
		}
	    }
	    Subscription current = new Subscription();
	    current.filter=filter;
	    current.handler=handler;
	    current.args=args;
	    subscriptions.add(current);
	}
    }
    public IndAction makePopup(Vector popupArgs) {
	System.out.println("\nwithin makePopup !");
	String args[] = new String[popupArgs.size()];
	popupArgs.copyInto(args);
	PopupAction popup = new PopupAction(args);
	System.out.println("\nabout to return popup !");
	return popup;
    }
    public static void main(String args[]) {
	try {
	    ProcessSubscriptions ps = new ProcessSubscriptions();
	    ps.parseSubscriptions();
	    System.out.println("Namespace: " + ps.namespace);
	    Enumeration allsubs = ps.subscriptions.elements();
	    while(allsubs.hasMoreElements()) {
		Subscription sub = (Subscription)allsubs.nextElement();
		System.out.println("  Filter: " + sub.filter);
		System.out.println("  Handler: " + sub.handler);
		System.out.println("  Args: " + sub.args);
	    }
	    CIMNameSpace cns = new CIMNameSpace(args[0],ps.namespace);
	    CIMClient cc = new CIMClient(cns,args[1],args[2]);
	    IndicationArbiter ia = new IndicationArbiter(cc);
	    System.out.println("Successfully started Indication Arbiter for " +
			       cns);
	    allsubs = ps.subscriptions.elements();
	    while(allsubs.hasMoreElements()) {
		Subscription sub = (Subscription)allsubs.nextElement();
		IndActionConsumer ac = null;
		if (sub.handler.equalsIgnoreCase("PopupAction")) {
		    ac = new IndActionConsumer(ps.makePopup(sub.args));
		} 
		ia.subscribe(new IndSubscription(cns,sub.filter,ac));
		System.out.println("Added new subscription");
	    }
	} catch (Exception exc) {
	    exc.printStackTrace();
	}
    }
}
