/*
 * sig_send.c
 *
 * (C) Copyright IBM Corp. 2004
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 *    Author: Hien Q Nguyen (hien@us.ibm.com)
 *    Contributors : Rajarshi Das (rajarshi@in.ibm.com)
 *
 * Interface Type: Common Manageability Programming Interface ( CMPI )
 *
 * Description: Test program to manually send signals to the daemon.
 */


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>

#include <signal.h>           	
#include <errno.h>
#include <assert.h>
#include <grp.h>
#include <pwd.h>
#include <sys/wait.h>
#include <sys/syscall.h>


void
usage()
{
	fprintf(stderr, "Usage: sig_send -p <pid> -r <recid>\n");
	exit(1);
}
int 
main(int argc, char *argv[])
{
	auto int c;
	int recid = -1;
	pid_t pid = 0;
	int sub_id = 234;		/* just an extra field, that we can use */
	struct siginfo si;
	union sigval val;
	int ret;
	
	if (argc < 2) {
		usage();
	}

	while ((c = getopt(argc, argv, "hp:r:")) != EOF) {
		switch (c) {
			case 'p':
				pid = atoi(optarg);
				break;
			case 'r':
				recid = atoi(optarg);
				break;	
			case 'h':
				usage();
				break;
		}
	}

	if (pid == 0 || recid == -1) {
		usage();
	}
	printf("pid=%d, recid=%d\n", pid, recid);
	
	si.si_code = SI_QUEUE;
	//si.si_signo = SIGRTMIN+1;
	si.si_signo = SIGRTMIN+14;
	si.si_errno = recid;	

	/* 
	 * Notice the sub id code, in the sig_recv
	 * We can use this to pass some extra stuff
	 * if we want to
	 */
	memcpy(&si._sifields._rt.si_sigval, &sub_id, sizeof(sigval_t));

	if ((ret =syscall(SYS_rt_sigqueueinfo, pid, si.si_signo, &si)) != 0) {
		if (ret == -ESRCH) {
			fprintf(stderr, "Error: No such process\n");
			exit(1);
		}
		fprintf(stderr, "SYS_rt_sigqueueinfo return %d\n", ret);
		exit(1);
	}
			
	return 0;
}

	
